
local WaterMill = ...

function registerMill(_nameFbx, _nameAsset, _nameJob, _nameResource, _nameProduction) 
    -- Register core prefab nodes (currently only one core model)
    WaterMill:registerAssetId("models/waterMill.fbx/Prefab/waterMill/Core" .. _nameFbx .. "Part", "WATER_MILL_CORE_" .. _nameAsset .. "_PREFAB")

    -- Register the water mill Core part
    WaterMill:register({
	DataType = "BUILDING_PART",
	Id = "WATER_MILL_CORE_" .. _nameAsset .. "_PART",
	Name = "WATER_MILL_CORE_" .. _nameAsset .. "_NAME",
	Description = "WATER_MILL_CORE_" .. _nameAsset .. "_DESC",
	Category = "CORE",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = "WATER_MILL_CORE_" .. _nameAsset .. "_PREFAB"
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 2 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "WOOD", Quantity = 5 }
		}
	},
	AssetBuildingFunction = "WATER_MILL_FUNCTION_CORE_" .. _nameAsset
    })

    WaterMill:register({
		DataType = "BUILDING_FUNCTION_WORKPLACE",
                Id = "WATER_MILL_FUNCTION_CORE_" .. _nameAsset,
		WorkerCapacity = 2,
                RelatedJob = { Job = _nameJob, Behavior = "BEHAVIOR_WORK" },
                InputInventoryCapacity = {
			{ Resource = _nameResource, Quantity = 10 }
                },
                ResourceListNeeded = {
			{ Resource = _nameResource, Quantity = 1 }
                },
                ResourceProduced = {
			{ Resource = _nameProduction, Quantity = 1 }
                }
    })

    -- Register grass clearing components
    WaterMill:registerPrefabComponent("models/waterMill.fbx/Prefab/waterMill/Core" .. _nameFbx .. "Part", {
	DataType = "COMP_DIRT_RECTANGLE",
	Size = {4, 8}
    })

    -- Register particle system
    WaterMill:registerAssetId("models/waterMill.fbx/Prefab/waterMill/Core" .. _nameFbx .. "Part/step_0/SmokeEmitter", "WATER_MILL_CORE_" .. _nameAsset .. "_SMOKE_EMITTER_PREFAB")

    WaterMill:registerPrefabComponent("models/waterMill.fbx/Prefab/waterMill/Core" .. _nameFbx .. "Part/step_0/SmokeEmitter", {
	DataType = "COMP_PARTICLE_EMITTER",
	ParticleSystem = "PARTICLE_SYSTEM_HOUSING_SMOKE",
	IsPlaying = true,
	IsEmitting = false
    })

    WaterMill:registerPrefabComponent("models/waterMill.fbx/Prefab/waterMill/Core" .. _nameFbx .. "Part/step_0/SmokeEmitter", {
	DataType = "COMP_PARTICLE_EMITTER_TOGGLE",
	Enabled = false
    })
end

registerMill( "Flour", "FLOUR", "MILLER", "WHEAT", "FLOUR")
registerMill( "Textile", "TEXTILE", "WEAVER", "WOOL", "WOOLEN_CLOTH")
registerMill( "Planks", "PLANKS", "CARPENTER", "WOOD", "PLANK")

WaterMill:registerPrefabComponent("models/waterMill.fbx/Prefab/waterMill/CoreFlourPart", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 4, 8 },
	FeedbackComponentListToActivate = {
		{ "WATER_MILL_CORE_FLOUR_SMOKE_EMITTER_PREFAB", "COMP_PARTICLE_EMITTER_TOGGLE" }
	}
})

WaterMill:registerPrefabComponent("models/waterMill.fbx/Prefab/waterMill/CoreTextilePart", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 4, 8 },
	FeedbackComponentListToActivate = {
		{ "WATER_MILL_CORE_TEXTILE_SMOKE_EMITTER_PREFAB", "COMP_PARTICLE_EMITTER_TOGGLE" }
	}
})

WaterMill:registerAssetId("models/waterMill.fbx/Prefab/waterMill/CorePlanksPart/step_0/SawSmokeEmitter", "WATER_MILL_CORE_PLANKS_SAW_SMOKE_EMITTER_PREFAB")

WaterMill:registerAssetId("models/waterMill.fbx/Prefab/waterMill/CorePlanksPart/step_0/SawSparkEmitter", "WATER_MILL_CORE_PLANKS_SAW_SPARK_EMITTER_PREFAB")


WaterMill:registerPrefabComponent("models/waterMill.fbx/Prefab/waterMill/CorePlanksPart", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 4, 8 },
	FeedbackComponentListToActivate = {
		{ "WATER_MILL_CORE_PLANKS_SMOKE_EMITTER_PREFAB", "COMP_PARTICLE_EMITTER_TOGGLE" },
		{ "WATER_MILL_CORE_PLANKS_SAW_SMOKE_EMITTER_PREFAB", "COMP_PARTICLE_EMITTER_TOGGLE" },
		{ "WATER_MILL_CORE_PLANKS_SAW_SPARK_EMITTER_PREFAB", "COMP_PARTICLE_EMITTER_TOGGLE" }
	}
})

WaterMill:registerPrefabComponent("models/waterMill.fbx/Prefab/waterMill/CorePlanksPart/step_0/SawSmokeEmitter", {
	DataType = "COMP_PARTICLE_EMITTER",
	ParticleSystem = "PARTICLE_SYSTEM_SAWMILL_SMOKE",
	IsPlaying = true,
	IsEmitting = false
})

WaterMill:registerPrefabComponent("models/waterMill.fbx/Prefab/waterMill/CorePlanksPart/step_0/SawSmokeEmitter", {
	DataType = "COMP_PARTICLE_EMITTER_TOGGLE",
	Enabled = false
})

WaterMill:registerPrefabComponent("models/waterMill.fbx/Prefab/waterMill/CorePlanksPart/step_0/SawSparkEmitter", {
	DataType = "COMP_PARTICLE_EMITTER",
	ParticleSystem = "PARTICLE_SYSTEM_SAWMILL_SPARK",
	IsPlaying = true,
	IsEmitting = false
})

WaterMill:registerPrefabComponent("models/waterMill.fbx/Prefab/waterMill/CorePlanksPart/step_0/SawSparkEmitter", {
	DataType = "COMP_PARTICLE_EMITTER_TOGGLE",
	Enabled = false
})

-- Components
--WaterMill:dofile("scripts/compWheel.lua")

function registerWheel(_nameFbx, _nameAsset, _reverseRotation)
    -- Register extension wheel prefab nodes
    WaterMill:registerAssetId("models/waterMill.fbx/Prefab/waterMill/Extension" .. _nameFbx .. "Part", "WATER_MILL_EXTENSION_" .. _nameAsset .. "_PREFAB")
    WaterMill:registerAssetId("models/waterMill.fbx/Prefab/waterMill/Extension" .. _nameFbx .. "Part/step_8_roue", "WATER_MILL_EXTENSION_" .. _nameAsset .. "WHEEL_PREFAB")

    -- Register the water mill Extension Wheel part
    WaterMill:register({
	DataType = "BUILDING_PART",
	Id = "WATER_MILL_EXTENSION_" .. _nameAsset .. "_PART",
	Name = "WATER_MILL_EXTENSION_" .. _nameAsset .. "_NAME",
	Description = "WATER_MILL_EXTENSION_" .. _nameAsset .. "_DESC",
	Category = "EXTENSION",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = "WATER_MILL_EXTENSION_" .. _nameAsset .. "_PREFAB",
                IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "WOOD", Quantity = 5 }
		}
	}
    })

    WaterMill:registerPrefabComponent("models/waterMill.fbx/Prefab/waterMill/Extension" .. _nameFbx .. "Part", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 4, 6 }
    })

    WaterMill:registerPrefabComponent("models/waterMill.fbx/Prefab/waterMill/Extension" .. _nameFbx .. "Part", {
	DataType = "COMP_GROUNDED",
	GroundToWater = true
    })

    -- Register air vent particle system (flow)
    WaterMill:registerPrefabComponent("models/waterMill.fbx/Prefab/waterMill/Extension" .. _nameFbx .. "Part/FlowEmitter", {
	DataType = "COMP_PARTICLE_EMITTER",
	ParticleSystem = "WATER_MILL_FLOW_PARTICLE_SYSTEM",
	IsPlaying = true,
	IsEmitting = true
    })
--[[
    WaterMill:registerPrefabComponent("models/waterMill.fbx/Prefab/waterMill/Extension" .. _nameFbx .. "Part", {
	DataType = "COMP_BUILDING_PART",
	FeedbackComponentListToActivate = {
		{ "WATER_MILL_EXTENSION_" .. _nameAsset .. "WHEEL_PREFAB", "WATER_MILL_COMP_WHEEL" }
	}
    })

    --------------------------------------------------------------------------
    -- Register wheel component
    WaterMill:registerPrefabComponent("models/waterMill.fbx/Prefab/waterMill/Extension" .. _nameFbx .. "Part/step_8_roue", {
	DataType = "WATER_MILL_COMP_WHEEL",
	Enabled = false,
        RotationDirectionCustom = _nameAsset
    })
--]]
end

registerWheel("LeftBank", "LEFT_BANK", false)
registerWheel("RightBank", "RIGHT_BANK", true)
